@@ -10,18 +10,20 @@ Type some short-hand, see the corresponding SQL and its output:
  select * from TABLE1 where somecolumn > 2;
  prepare: 0.000s   execute: 0.073s   rows: 14
    id|column1                                    |column2
-     |                                           |    |somecolumn
- ----|-------------------------------------------|----|-|
-   27|foo                                        |    |7|
-   49|bar                                        |abcd|3|
-   81|baz\nbazinga\nbazurka                      |jk  |9|
- 1984|bla bla bla bla bla bla bla bla bla bla bla|xyz |5|
+     |                                           |    |column3
+     |                                           |    | |somecolumn
+ ----|-------------------------------------------|----|-|-|
+   27|foo                                        |    |a|7|
+   49|bar                                        |abcd|a|3|
+   81|baz\nbazinga\nbazurka                      |jk  |b|9|
+ 1984|bla bla bla bla bla bla bla bla bla bla bla|xyz |c|5|
  ...
  steno> /abc|foo/#TBL1;.socol > 2    -- /regexp/ grep, #tableabbrev, .columnabbrev
  select * from TABLE1 where somecolumn > 2;
  prepare: 0.000s   execute: 0.039s   rows: 14
  id|column1
    |   |column2
+   |   |    |[column3=a]
    |   |    |somecolumn
  --|---|----|-|
  27|foo|    |7|
@@ -63,7 +65,7 @@ with other DBs.
 use v5.10;
 
 package SQL::Steno;
-our $VERSION = 'v0.1.1';	# quotes because of CPAN indexer bug
+our $VERSION = 'v0.1.2';	# quotes because of CPAN indexer bug
 
 use strict;
 use Time::HiRes qw(gettimeofday tv_interval);
@@ -72,7 +74,7 @@ binmode STDIN, ':utf8';
 binmode STDOUT, ':utf8';
 
 our $dbh;
-
+our $perl_re = qr/(\{(?:(?>[^{}]+)|(?-1))*\})/;
 our( %Table_Columns, $table_re );
 sub init {
     die "\$dbh is undef\n" unless $dbh;
@@ -349,7 +351,7 @@ sub convert_table_column {
 	    undef $column{$_} for @{$Table_Columns{$_}};
 	}
 	my @column = keys %column;
-	s/(^|[-+\s(,;&|])?(?<!\\)\.([a-z]\w*)/(defined $1 ? $1 : '.') . find $2, '.', '', %Columns, @column/egi;
+	s/(^|[-+\s(,;&|])?(?<!\\)\.([a-z]\w*)(?:\.(\w*))?/(defined $1 ? $1 : '.') . (find $2, '.', '', %Columns, @column) . ($3 ? " $3" : defined $3 ? " $2" : '')/egi;
     }
 }
 
@@ -410,11 +412,14 @@ offset.  The letters are:
 
 Then it looks for C<#tbl>, C<#tbl#> or C<#tbl#alias>.  Here tbl is a key of
 C<%Tables> or any abbreviation of known tables in C<@Tables>.  If followed by
-C<#>, the abbrev is used as an alias, unless an alias directly follows, in
-which case that is used.
+C<#>, the abbreviation is used as an alias, unless an alias directly follows,
+in which case that is used.
 
-Then it looks for C<.col>.  Here col is a key of C<%Columns> or any
-abbreviation of columns of any table recognized in the query.
+Then it looks for C<.col>, C<.col.> C<.col.alias>.  Here col is a key of
+C<%Columns> or any abbreviation of columns of any table recognized in the
+query.  If followed by C<.>, the abbreviation is used as an alias, unless an
+alias directly follows, in which case that is used.  It tries to be clever
+about whether the 1st C<.> needs to be preserved, i.e. following a table name.
 
 Finally it looks for C<func(>.  Here func is a key of C<%Functions> or any
 abbreviation of known functions in C<@Functions>, which includes words
@@ -459,8 +464,12 @@ sub convert {
 	}
     }
 
-    # \todo (?(?<=\w)\b)
-    s&(?<!\\):($timespec_re[+-]\d*$timespec_re(?(?<=\w)\b)|l?j\w+(?:#(\w*))|\w+)&find $1, ':', '', %Macros&ego unless $error;
+    until( $error ) {
+	s&:$perl_re&my $ret = eval $1; warn $@ if $@; $ret // ' NULL '&ego or
+	    # \todo (?(?<=\w)\b)
+	    s&:($timespec_re[+-]\d*$timespec_re(?(?<=\w)\b)|l?j\w+(?:#(\w*))|\w+)&find $1, ':', '', %Macros&ego or
+		last;
+    }
 
     s&^(?=#)&;&;		# Assume empty fieldlist before table name
     &convert_table_column;
@@ -494,11 +503,11 @@ my %ctrl =
      "\r", '\r');
 my $lasttime = time;
 
-sub run($;$$$) {
-    my( $sql, $re, $fh, $csv ) = @_;
+sub run($;$$$$) {
+    my( $sql, $re, $sub, $fh, $csv ) = @_;
     my $t0 = [gettimeofday];
     if( $DBI::err || $t0->[0] - $lasttime > 3600 and !$dbh->ping ) {
-	printf STDOUT "Inactive for %ds, ping failed after %.03fs, your set variables are lost.\n",
+	printf STDOUT "Inactive for %ds, ping failed after %.03fs, your session variables are lost.\n",
 	    $t0->[0] - $lasttime, tv_interval $t0;
 	#$dbh->disconnect;
 	$dbh = $dbh->clone;	# reconnect
@@ -517,6 +526,7 @@ sub run($;$$$) {
 	    my @len = (1) x @name;
 	    my( @txt, @res, @comp );
 	    while( my @res1 = $sth->fetchrow_array ) {
+		next if $sub && !$sub->( \@res1, \@name );
 		next if $re && join( '|', map $_ // '\@', @res1 ) !~ $re;
 		if( $csv ) {
 		    for( @res1 ) {
@@ -631,14 +641,30 @@ order (i.e. C<&xyz> can return C</regexp/=literal sql> etc.):
 
 =item &query( arg, ... ) following text
 
+=item &{Perl code} following text
+
 These allow canned entries and are more complex than macros, in that they take
 arguments and replacement can depend on the argument.
 
-=item /regexp/...
+Run I<Perl code>.  It sees the I<following text> in C<$_> and may modify it.
+If it returns undef this statement is skipped.  If it returns a DBI statement
+handle run that instead of this statement.  Else replace with what it returns.
 
-This will treat the statement ... normally, but will join each output row with
-'|' characters and check that against the regexp.  Only matching output rows
-are considered.
+Reprocess result as a shell entry (i.e. it may return another C<&>query).
+
+=item {Perl code}statement
+
+Call I<Perl code> for every output row returned by the I<statement> with the
+array of values as first and the array of column names as second argument.  It
+may modify individual values, but not the array.  If it returns false, the row
+is skipped.
+
+=item /regexp/statement
+
+This will treat the I<statement> normally, but will join each output row with
+'|' characters (which need to be matched as C<\/>) and check that against the
+regexp.  Only matching rows are considered.  You may combine S<C<{Perl
+code}/regexp/>>.
 
 =item =literal sql
 
@@ -657,18 +683,20 @@ matching abbrev or C<?#abbrev> only those tables matching abbrev.
 Will convert and show, but not perform statement.  If C<$dbh> is not set, this
 is the default behaviour.
 
-=item !Unix Shell code
+=item !System Shell code
 
 Run it.
 
-=item {Perl code}
-
-Run it.  If it returns a DBI statement handle also run that.
-
 =item >filename
 
-Redirect next statement to filename.  If it has a suffix C<.csv>, an according
-format is used.
+=item >>filename
+
+Redirect or append next statement's output to I<filename>.  If it has a suffix
+C<.csv>, an according format is used.
+
+=item |System Shell code
+
+Pipe next statement's output through I<System Shell code>.
 
 =back
 
@@ -695,10 +723,33 @@ sub shell() {
 	}
 	s/\s+\Z//;
 
-	s!^\&(\w+)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Queries $1, $3!e
-	    or s!^\&(\w+) *(.*)!convert_Queries $1, $2!e unless $error;
+	until( $error ) {
+	    last unless
+		s!^\&(\w+)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Queries $1, $3!e
+		or s!^\&(\w+) *(.*)!convert_Queries $1, $2!e
+		or my $perl = s!^\&$perl_re!!;
+	    if( $perl ) {
+		$perl = eval $1;
+		local $| = 1;	# flush to avoid stderr prompt overtaking last output line.
+		warn $@ if $@;
+		if( UNIVERSAL::isa $perl, 'DBI::st' ) {
+		    $_ = $perl;
+		    goto RUN;
+		} elsif( defined $perl ) {
+		    substr $_, 0, 0, $perl;
+		} else {
+		    goto NEXT;
+		}
+	    }
+	}
 
-	my $re = $1 ? qr/$2/i : qr/$2/ if s!^(i?)/(.+?)/\s*!!;
+	my $sub;
+	if( s/^\s*$perl_re// ) {
+	    $sub = eval "sub $1";
+	    warn $@ if $@;
+	}
+	my $re = $1 ? qr/$2/i : qr/$2/
+	    if s!^(i?)/(.+?)/\s*!!;
 	goto RUN if s/^\s*=//;	# run literally
 
 	my $skip = 0;
@@ -723,20 +774,14 @@ sub shell() {
 	    }
 	    goto NEXT;
 	}
-	if( s/^\s*(\{(?:(?>[^{}]+)|(?1))*\})// ) {
-	    my $ret = eval $1;
-	    warn $@ if $@;
-	    if( UNIVERSAL::isa $ret, 'DBI::st' ) {
-		$_ = $ret;
-		goto RUN;
-	    }
-	    local $| = 1;	# flush to avoid stderr prompt overtaking last output line.
-	    goto NEXT;
-	}
 	s/^\s*()//;			# dummy because $1 survives loop iterations :-o
-	if( /\A>\s*(.+?(\.csv)?)\s*\Z/ ) {	# redirect output
-	    $csv = 1 if $2;
-	    open $fh, '>:utf8', (glob $1)[0];
+	if( /\A(>{1,2})\s*(.+?(\.csv)?)\s*\Z/ ) {	# redirect output
+	    $csv = 1 if $3;
+	    open $fh, "$1:utf8", (glob $2)[0];
+	    select $fh;
+	    goto NEXT;
+	} elsif( /\A\|(.+)\Z/ ) {	# pipe output
+	    open $fh, '|-:utf8', $1;
 	    select $fh;
 	    goto NEXT;
 	}
@@ -749,7 +794,7 @@ sub shell() {
 	goto NEXT if $skip;
 
       RUN:
-	run $_, $re, $fh, $csv if $dbh;
+	run $_, $re, $sub, $fh, $csv if $dbh;
 	if( $fh ) {
 	    close;
 	    select STDOUT;
@@ -872,7 +917,7 @@ the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<DBI>, L<SQL::Interp>, L<SQL::Preproc>, L<SQL::Yapp>
+L<DBI>, L<SQL::Interp>, L<SQL::Preproc>, L<SQL::Yapp>, L<Jade|http://jade-lang.com/>
 
 =head1 AUTHOR
 
